#include "audio.h"
#include "tpt_svf.h"
#include "fuzz.h"
#include "delay.h"
#include "reverb.h"

static tpt_svf_s svf1;
static tpt_svf_s svf_osc;
static varesky_lfo s_lfo;

static varesky_delay del;
static float delbuf[16384] = {0};

static float lfo = 0;
/// static float lfoSpeed;

void setupDSP(int buffersize, float samplerate)
{
    tpt_svf_setup(&svf1, samplerate);
    tpt_svf_setCutoff(&svf1, 1500);
    tpt_svf_setRes(&svf1, 0.2);

    tpt_svf_setup(&svf_osc, samplerate);
    tpt_svf_setCutoff(&svf_osc, 0.3);
    tpt_svf_setRes(&svf_osc, 1);

    v_lfo_init(&s_lfo, samplerate, 0.3, 0);

    varesky_del_setup(&del, &delbuf, sizeof(delbuf) / sizeof(float), 1234);
    varesky_del_set_delay(&del, 13000);
    del.fdb = 0.38;
    del.wet = 0.4;

    varesky_plate_init(samplerate);
}

void audioDSPCallback(float *audio, user_controls *controls, int size)
{

    static int i = 0;

    //FUZZ
    if (controls->bypass & BYP_FUZZ)
    {
        for (i = 0; i < size; i++)
        {
            audio[2 * i] = 0.25 * FUZZ(audio[2 * i]);
            audio[2 * i + 1] = 0.25 * FUZZ(audio[2 * i + 1]);
        }
    }

    //DELAY
    if (controls->bypass & BYP_DELAY)
    {
        //del.delay = controls->delayTime;
        del.fdb = controls->feedback;
        
        del.wet = controls->del_mix;
        del.dry = 1 - del.wet;
        v_lfo_set_rate(&s_lfo, controls->mod_rate);

        for (i = 0; i < size; i++)
        {
            // process the audio
            audio[2 * i] = varesky_del_process(&del, audio[2 * i]); // tmp;     // left
            audio[2 * i + 1] = audio[2 * i];                        // tmp; // right

            // lfo on read pointer
            lfo = v_lfo_process(&s_lfo) * controls->modulation;

            varesky_del_set_delay(&del, controls->delayTime + lfo * 1000);
        }
    }

    //REVERB
    if (controls->bypass & BYP_REVERB)
    {
        varesky_plate_set_decay(controls->rev_decay);
        varesky_plate_set_mix(controls->rev_mix);
        varesky_plate_set_damping(controls->rev_damping);
        varesky_plate_set_bandwidth(controls->rev_bandwidth);
        varesky_plate_set_predelay(controls->rev_predelay);
        varesky_plate_process_block(audio, audio, size);
    }

    // simulate heavy processing
    // int wr = 555;
    // for(i=0; i<100000; i++)wr = (wr + 100)&666;
}
