#include <math.h>
#define tpt_svf_pi (3.1415926535897932384626433832795028841)

#include "tpt_svf.h"

void tpt_svf_calc(tpt_svf_s *s)
{
    s->g = tan(tpt_svf_pi * s->cutoff / s->samplerate);
    // k = 1/Q = 2-2*res;
    s->k = 2 - 2 * s->res;
    s->a1 = 1 / (1 + s->g * (s->g + s->k));
    s->a2 = s->g * s->a1;
}

void tpt_svf_setup(tpt_svf_s *s, int samplerate)
{
    s->ic1eq = 0;
    s->ic2eq = 0;
    s->samplerate = samplerate;
    s->cutoff = 1000;
    s->res = 0.3;
    tpt_svf_calc(s);
}
void tpt_svf_setCutoff(tpt_svf_s *s, float cutoff)
{
    s->cutoff = cutoff;
    tpt_svf_calc(s);
}

void tpt_svf_setRes(tpt_svf_s *s, float res)
{
    s->res = res;
    tpt_svf_calc(s);
}

void tpt_svf_setOutput(tpt_svf_s *s)
{

}

float tpt_svf_process_sample(tpt_svf_s *s, double x)
{

    s->v1 = s->a1 * s->ic1eq + s->a2 * (x - s->ic2eq);
    s->v2 = s->ic2eq + s->g * s->v1;
    s->ic1eq = 2 * s->v1 - s->ic1eq;
    s->ic2eq = 2 * s->v2 - s->ic2eq;

    return s->v1;
}

void tpt_svf_process_block(tpt_svf_s *s, double *data, int size)
{
    static int i;
    for (i = 0; i < size; i++)
    {
        s->v1 = s->a1 * s->ic1eq + s->a2 * (data[i] - s->ic2eq);
        s->v2 = s->ic2eq + s->g * s->v1;
        s->ic1eq = 2 * s->v1 - s->ic1eq;
        s->ic2eq = 2 * s->v2 - s->ic2eq;

        // interleaved output
        data[2 * i] = s->v1;
        data[2 * i + 1] = s->v1;
    }
}
