/*
 * Copyright (c) 2013-2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 *  ======== App.c ========
 *
 */

/* host header files */
#include <stdio.h>
#include <unistd.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include <sys/time.h>
#include <time.h>

#include <ti/cmem.h>
#include <_bigdataxlat.h>

/* package header files */
#include <ti/ipc/Std.h>
#include "Std.h"
#include <ti/ipc/MessageQ.h>
#include "HeapMem.h"
#include "SharedRegion.h"
#include "MemoryDefs.h"

/* local header files */
#include "../shared/AppCommon.h"
#include "App.h"

#include "ARM_audio.h"
#include "reverb.h"

/* Application specific defines */
#define BIG_DATA_POOL_SIZE 0x1000000

/* round up the value 'size' to the next 'align' boundary */
#define ROUNDUP(size, align) \
    (UInt32)(((UInt32)(size) + ((UInt32)(align)-1)) & ~((UInt32)(align)-1))

/* module structure */
typedef struct
{
    MessageQ_Handle hostQue;   // created locally
    MessageQ_QueueId slaveQue; // opened remotely
    UInt16 heapId;             // MessageQ heapId
    UInt32 msgSize;
} App_Module;

/* private data */
static App_Module Module;

static Shared_mem *shmem;

static Int status;
static App_Msg *msg;
static UInt32 *bigDataLocalPtr;
static Int i, j;
static UInt16 regionId1 = 1;
static SharedRegion_Entry *pSrEntry;
static void *sharedRegionAllocPtr = NULL;
static Int pool_id;
static CMEM_AllocParams cmemAttrs;
static HeapMem_Handle sr1Heap;
static UInt16 regionId;
static UInt32 errorCount = 0;
static Int retVal;
static bigDataLocalDesc_t bigDataLocalDesc;
static SharedRegion_Config sharedRegionConfig;
static SharedRegion_Entry srEntry;
static HeapMem_Config HeapMemConfig;
static HeapMem_Params heapMem_params;
static Memory_Stats stats;
static HeapMem_ExtendedStats extStats;

static clock_t start;
static clock_t end;
static double elapsed;

struct timeval tv;
struct timeval start_tv;
static double average = 0, max = 0;
unsigned int cnt = 0;

/*
 *  ======== App_create ========
 */
static user_controls local_controls;

void init_user_controls()
{
    local_controls.bypass = 0;
    // local_controls.bypass &= BYP_REVERB;
    // local_controls.bypass &= BYP_DELAY;
    local_controls.bypass |= BYP_DSP;

    local_controls.rev_decay = 0.8;
    local_controls.rev_mix = 0.707;
    local_controls.rev_damping = 0.2;
    local_controls.rev_bandwidth = 0.6;
    local_controls.rev_mix = 0.5;

    local_controls.delayTime = 10000;
    local_controls.feedback = 0.777;
    local_controls.modulation = 0.1;
    local_controls.mod_rate = 0.53;
    local_controls.del_mix = 0.5;
}

static int choice;
static bool dspState = 1;

void preset_menu()
{
    while (1)
    {
        printf("\e[1;1H\e[2J");
        printf("Presets menu:\n0)Back\n1)Endless reverb\n2)Chorus\n3)Simple delay\n4)Distorted lush\n5)Dark plate\n6)Default\n");
        scanf("%d", &choice);

        switch (choice)
        {
        case 0:
            printf("Quitting...");
            return;
        case 1:
            printf("Quitting...");
            local_controls.rev_decay = 1;
            local_controls.rev_bandwidth = 0.999;
            local_controls.rev_damping = 0;
            local_controls.rev_mix = 0.84;
            local_controls.rev_predelay = 123.321;
            local_controls.bypass = BYP_DSP | BYP_REVERB;
            break;
        case 2:
            printf("Quitting...");
            local_controls.delayTime = 800;
            local_controls.modulation = 0.3;
            local_controls.mod_rate = 0.533;
            local_controls.del_mix = 0.707;
            local_controls.feedback = 0;

            local_controls.bypass = BYP_DSP | BYP_DELAY;

            break;
        case 3:
            printf("Quitting...");
            local_controls.delayTime = 15000;
            local_controls.modulation = 0;
            local_controls.mod_rate = 0;
            local_controls.del_mix = 0.407;
            local_controls.feedback = 0.64432;

            local_controls.bypass = BYP_DSP | BYP_DELAY;

            break;
        case 4:
            local_controls.rev_decay = 0.677;
            local_controls.rev_bandwidth = 0.7;
            local_controls.rev_damping = 0.2;
            local_controls.rev_mix = 0.5;
            local_controls.rev_predelay = 1.321;

            local_controls.delayTime = 3333;
            local_controls.modulation = 0.4;
            local_controls.mod_rate = 0.433;
            local_controls.del_mix = 0.707;
            local_controls.feedback = 0.1;

            local_controls.bypass = BYP_DSP | BYP_DELAY | BYP_FUZZ | BYP_REVERB;

            break;
        case 5:
            local_controls.rev_decay = 0.3;
            local_controls.rev_bandwidth = 0.23445;
            local_controls.rev_damping = 0.7;
            local_controls.rev_mix = 0.4;
            local_controls.rev_predelay = 12.321;
            local_controls.bypass = BYP_DSP | BYP_REVERB;
            break;

        case 6:
            init_user_controls();
            break;
        default:
            break;
        }
    }
}
void reverb_menu()
{
    while (1)
    {
        printf("\e[1;1H\e[2J");
        printf("Reverb parameter menu:\n0)Back\n1)Toggle bypass\n2)Decay\n3)Mix\n4)Bandwidth\n5)Damping\n6)Predelay\n");
        scanf("%d", &choice);

        switch (choice)
        {
        case 0:
            printf("Quitting...");
            return;
            break;
        case 1:
            printf("\e[1;1H\e[2J");
            local_controls.bypass ^= BYP_REVERB;

            break;
        case 2:
            printf("\e[1;1H\e[2J");
            printf("Enter decay parameter (0-1): ");
            scanf("%f", &local_controls.rev_decay);
            break;
        case 3:
            printf("\e[1;1H\e[2J");
            printf("Enter mix parameter (0-1): ");
            scanf("%f", &local_controls.rev_mix);
            break;
        case 4:
            printf("\e[1;1H\e[2J");
            printf("Enter bandwidth parameter (0-1): ");
            scanf("%f", &local_controls.rev_bandwidth);
            break;
        case 5:
            printf("\e[1;1H\e[2J");
            printf("Enter damping parameter (0-1): ");
            scanf("%f", &local_controls.rev_damping);
            break;
        case 6:
            printf("\e[1;1H\e[2J");
            printf("Enter predelay parameter (1-170 ms): ");
            scanf("%f", &local_controls.rev_predelay);
            break;
        default:
            break;
        }
    }
}
void delay_menu()
{
    while (1)
    {
        printf("\e[1;1H\e[2J");
        printf("Delay parameter menu:\n0)Back\n1)Toggle bypass\n2)Delay time\n3)Feedback\n4)Modulation\n5)Mod rate\n6)Mix\n");
        scanf("%d", &choice);

        switch (choice)
        {
        case 0:
            printf("Quitting...");
            return;
            break;
        case 1:
            printf("\e[1;1H\e[2J");
            local_controls.bypass ^= BYP_DELAY;
            break;
        case 2:
            printf("\e[1;1H\e[2J");
            printf("Enter delay time parameter (1001-15383 samps): ");
            scanf("%f", &local_controls.delayTime);
            break;
        case 3:
            printf("\e[1;1H\e[2J");
            printf("Enter feedback time parameter (0-1): ");
            scanf("%f", &local_controls.feedback);
            break;
        case 4:
            printf("\e[1;1H\e[2J");
            printf("Enter modulation parameter (0-1): ");
            scanf("%f", &local_controls.modulation);
            break;
        case 5:
            printf("\e[1;1H\e[2J");
            printf("Enter mod rate parameter (0-1): ");
            scanf("%f", &local_controls.mod_rate);
            break;
        case 6:
            printf("\e[1;1H\e[2J");
            printf("Enter mix parameter (0-1): ");
            scanf("%f", &local_controls.del_mix);
            break;
        default:
            break;
        }
    }
}
void ui_menu()
{

    while (1)
    {
        printf("\e[1;1H\e[2J");
        printf("Audio parameter menu:\n0)Quit\n1)Reverb\n2)Delay\n3)Fuzz toggle\n4)DSP BYPASS %d\n5)Presets\n", dspState);
        scanf("%d", &choice);

        switch (choice)
        {
        case 0:
            printf("Quitting...");
            return;
            break;
        case 1:
            reverb_menu();
            break;
        case 2:
            delay_menu();
            break;
        case 3:
            local_controls.bypass ^= BYP_FUZZ;
            break;
        case 4:
            local_controls.bypass ^= BYP_DSP;
            dspState = !dspState;
            break;
        case 5:
            preset_menu();
            break;
        default:
            break;
        }
    }
}

static UInt32 msgIds = 0;
static unsigned int underflows = 0;
static int writePos = 0, readPos = 1, dspPos = 0;
static int nMessages = 0;

static struct timespec t0, t1;
double us_diff ( struct timespec *t1, struct timespec *t2)
{ return (&t2->tv_sec - &t1->tv_sec) * 1e6 + (&t2->tv_nsec - &t1->tv_nsec) / 1e3; }

int audioCallback(const void *inputBuffer, void *outputBuffer,
                  unsigned long framesPerBuffer,
                  const PaStreamCallbackTimeInfo *timeInfo,
                  PaStreamCallbackFlags statusFlags,
                  void *userData)
{
    //gettimeofday(&start_tv, NULL);
    clock_gettime(CLOCK_MONOTONIC, &t0);

    float *out = (float *)outputBuffer;
    const float *in = (const float *)inputBuffer;


    memcpy(shmem->dataBuf[writePos], in, sizeof(float) * 2 * framesPerBuffer);
    Cache_wb(bigDataLocalDesc.localPtr, bigDataLocalDesc.size, Cache_Type_ALL, TRUE);
    // /////////////////////////////////////////////////////////////////send message

    msg = (App_Msg *)MessageQ_alloc(Module.heapId, Module.msgSize);

    if (msg == NULL)
    {
        status = -1;
        printf("MessageQ_alloc failed\n");
        // goto leave;
        return paContinue;
    }

    MessageQ_setReplyQueue(Module.hostQue, (MessageQ_Msg)msg);
    msg->cmd = App_CMD_NEWDATA;
    msg->id = msgIds;
    // msg->rd = shmem->wr;
    msg->rd = writePos;

    MessageQ_put(Module.slaveQue, (MessageQ_Msg)msg);


    if (MessageQ_get(Module.hostQue, (MessageQ_Msg *)&msg, MessageQ_FOREVER) < 0)
    {
        printf("error receiving\n");
        return paContinue;
    }


    /////////////////////////////////////////////////////////////////////////received
    writePos = (writePos + 1) & (N_BUFFS - 1);
    // dspPos = msg->rd;

    memcpy(out, shmem->dataBuf[readPos], sizeof(float) * 2 * framesPerBuffer);
    readPos = (readPos + 1) & (N_BUFFS - 1);
    Cache_inv(bigDataLocalDesc.localPtr, bigDataLocalDesc.size, Cache_Type_ALL, TRUE);
    // ////////////////////////////////////////////////////////////////////copy data back
    // msgIds++;

    MessageQ_free((MessageQ_Msg)msg);

    //gettimeofday(&tv, NULL);
    clock_gettime(CLOCK_MONOTONIC, &t1);
    //elapsed = (t1.tv_nsec - t0.tv_nsec) * 1e3;//us_diff(&t0,&t1);//(tv.tv_sec - start_tv.tv_sec) + 1e-3 * (tv.tv_usec - start_tv.tv_usec);
    elapsed = (t1.tv_sec - t0.tv_sec) * 1e6 + (t1.tv_nsec - t0.tv_nsec) / 1e3;

    if (elapsed > 100)
    {
        cnt++;
        average += elapsed;
        if (elapsed > max)
            max = elapsed;
        printf("Time measured: %f useconds cnt: %d.\n", elapsed, cnt);
        if(cnt == 1000)return paAbort;
    }

    return paContinue;
}

Int App_create(UInt16 remoteProcId)
{
    Int status = 0;
    MessageQ_Params msgqParams;
    char msgqName[32];

    varesky_plate_init(SAMPLE_RATE);

    init_user_controls();

    printf("--> App_create:\n");

    /* setting default values */
    Module.hostQue = NULL;
    Module.slaveQue = MessageQ_INVALIDMESSAGEQ;
    Module.heapId = App_MsgHeapId;
    Module.msgSize = sizeof(App_Msg);

    /* create local message queue (inbound messages) */
    MessageQ_Params_init(&msgqParams);

    Module.hostQue = MessageQ_create(App_HostMsgQueName, &msgqParams);

    if (Module.hostQue == NULL)
    {
        printf("App_create: Failed creating MessageQ\n");
        status = -1;
        goto leave;
    }

    /* open the remote message queue */
    sprintf(msgqName, App_SlaveMsgQueName, MultiProc_getName(remoteProcId));

    do
    {
        status = MessageQ_open(msgqName, &Module.slaveQue);
        sleep(1);
    } while (status == MessageQ_E_NOTFOUND);

    if (status < 0)
    {
        printf("App_create: Failed opening MessageQ\n");
        goto leave;
    }

    printf("App_create: Host is ready\n");

leave:
    printf("<-- App_create:\n");
    return (status);
}

/*
 *  ======== App_delete ========
 */
Int App_delete(Void)
{
    Int status;

    if (AudioON)
    {
        stopAudio();
    }

    printf("--> App_delete:\n");

    /* close remote resources */
    status = MessageQ_close(&Module.slaveQue);

    if (status < 0)
    {
        goto leave;
    }

    /* delete the host message queue */
    status = MessageQ_delete(&Module.hostQue);

    if (status < 0)
    {
        goto leave;
    }

leave:
    printf("<-- App_delete:\n");
    return (status);
}

/*
 *  ======== App_exec ========
 */
Int App_exec(Void)
{

    printf("--> App_exec:\n");

    /* CMEM: contiguous memory manager for HLOS */
    /* initialised here */
    status = CMEM_init();

    if (status < 0)
    {
        printf("CMEM_init failed\n");
        goto leave;
    }
    else
    {
        printf("CMEM_init success\n");
    }

    pool_id = CMEM_getPool(BIG_DATA_POOL_SIZE);
    if (pool_id < 0)
    {
        printf("CMEM_getPool failed\n");
        goto leave;
    }
    printf("CMEM_getPool success\n");

    cmemAttrs.type = CMEM_HEAP;
    cmemAttrs.flags = CMEM_CACHED;
    cmemAttrs.alignment = 0;
    sharedRegionAllocPtr = CMEM_allocPool(pool_id, &cmemAttrs);
    if (sharedRegionAllocPtr == NULL)
    {
        printf("CMEM_allocPool failed\n");
        goto leave;
    }
    printf("CMEM_allocPool success: Allocated buffer %p\n", sharedRegionAllocPtr);

    /* Create shared region */
    SharedRegion_getConfig(&sharedRegionConfig);
    status = SharedRegion_setup(&sharedRegionConfig);
    if (status < 0)
    {
        printf("SharedRegion_setup failed\n");
        goto leave;
    }
    printf("SharedRegion_setup success\n");

    /* Configure srEntry */
    srEntry.base = sharedRegionAllocPtr;
    srEntry.len = BIG_DATA_POOL_SIZE;
    srEntry.ownerProcId = MultiProc_self();
    /* Make sure this is enabled if using Cached memory */
    srEntry.isValid = TRUE;
    srEntry.cacheEnable = TRUE;
    srEntry.createHeap = FALSE;
    srEntry.cacheLineSize = 128;
    srEntry.name = "SR1";

    status = SharedRegion_setEntry(regionId1, &srEntry);

    pSrEntry = SharedRegion_getEntryPtr(regionId1);
    printf("App_taskFxn: SR_1, base 0x%x, len=%x\n", (UInt32)pSrEntry->base, pSrEntry->len);

    regionId = regionId1;

    /* Setup HeapMem module */
    HeapMem_getConfig(&HeapMemConfig);
    status = HeapMem_setup(&HeapMemConfig);
    if (status < 0)
    {
        printf("HeapMem_setup failed\n");
        goto leave;
    }
    printf("HeapMem_setup success\n");

    /* Create HeapMP at run-time:
        This heap is intended to be used for big data ipc */
    HeapMem_Params_init(&heapMem_params);
    heapMem_params.name = "sr1HeapMem";
    heapMem_params.sharedAddr = pSrEntry->base;
    heapMem_params.sharedBufSize = ROUNDUP(pSrEntry->len, pSrEntry->cacheLineSize);
    heapMem_params.gate = NULL;
    sr1Heap = HeapMem_create(&heapMem_params);
    if (!sr1Heap)
    {
        printf("sr1Heap creation failed\n");
        status = -1;
        goto leave;
    }
    printf("HeapMem_create success\n");
    HeapMem_getStats((HeapMem_Handle)sr1Heap, &stats);
    printf("App_taskFxn: SR_1 heap, totalSize=%d,totalFreeSize=%d,largestFreeSize=%d\n", stats.totalSize, stats.totalFreeSize, stats.largestFreeSize);
    HeapMem_getExtendedStats((HeapMem_Handle)sr1Heap, &extStats);
    printf("App_taskFxn: SR_1 heap, buf=0x%p,size=%d\n", extStats.buf, extStats.size);

    shmem = (Shared_mem *)(HeapMem_alloc(sr1Heap, sizeof(Shared_mem), BIGDATA_BUF_ALIGN));
    // shmem->Buf_size = FRAMES_PER_BUFFER * N_CHANNELS;
    //  shmem->rd = 1;
    //  shmem->dsp_rd = 0;
    //  shmem->wr = 0;

    // for (j = 0; j < shmem->Buf_size; j++)
    // {
    //     // bigDataLocalPtr[j] = 666;
    //     // shmem->dataBuf[0][j] = 666;
    //     shmem->dataBuf[0][j] = 666;
    // }

    /* fill process pipeline */
    for (i = 1; i <= 3; i++)
    {
        printf("App_exec: sending message %d\n", i);

        /* allocate message */
        msg = (App_Msg *)MessageQ_alloc(Module.heapId, Module.msgSize);

        if (msg == NULL)
        {
            status = -1;
            goto leave;
        }

        /* set the return address in the message header */
        MessageQ_setReplyQueue(Module.hostQue, (MessageQ_Msg)msg);

        if (i == 1)
        {
            /* fill in message payload for Shared region init*/
            msg->cmd = App_CMD_SHARED_REGION_INIT;
            msg->id = i;
            msg->regionId = regionId;
            /* Passing the local shared memory address to the remote */
            /* Actually this can be any allocated buffer for the used for the heap */
            msg->u.sharedRegionInitCfg.base = CMEM_getPhys(pSrEntry->base);
            printf("Shared memory phys Addr %llx\n", msg->u.sharedRegionInitCfg.base);
            if (!msg->u.sharedRegionInitCfg.base)
            {
                printf("CMEM_getPhys failed\n");
            }
            msg->u.sharedRegionInitCfg.size = (UInt64)(pSrEntry->len);
        }
        else
        {
            /* fill in message payload */
            msg->cmd = App_CMD_NOP;
            msg->id = i;
        }

        /* send message */
        MessageQ_put(Module.slaveQue, (MessageQ_Msg)msg);
    }

    /* process steady state (keep pipeline full) */
    for (i = 4; i <= 16; i++)
    {

        /* Now this section of code starts receiving messages
           See the next section for the code for sending further messages */
        /* Receive messages: Start <======================================= */

        /* wait for return message */
        status = MessageQ_get(Module.hostQue, (MessageQ_Msg *)&msg,
                              MessageQ_FOREVER);

        if (status < 0)
        {
            goto leave;
        }

        /* extract message payload */

        if (msg->cmd == App_CMD_BIGDATA)
        {

            retVal = bigDataXlatetoLocalAndSync(msg->regionId,
                                                &msg->u.bigDataSharedDesc, &bigDataLocalDesc);
            if (retVal)
            {
                status = -1;
                goto leave;
            }
            shmem = (Shared_mem *)bigDataLocalDesc.localPtr;
#ifdef DEBUG
            /* print data from big data buffer */
            printf(" Received back buffer %d\n", msg->id);
            printf(" First 8 bytes: \n");
            for (j = 0; j < 8 && j < bigDataLocalDesc.size / sizeof(uint32_t); j += 4)
                printf("0x%x, 0x%x, 0x%x, 0x%x\n",
                       bigDataLocalPtr[j], bigDataLocalPtr[j + 1], bigDataLocalPtr[j + 2], bigDataLocalPtr[j + 3]);
            printf(" Last 8 bytes: \n");
            for (j = (bigDataLocalDesc.size / sizeof(uint32_t)) - 8;
                 j < bigDataLocalDesc.size / sizeof(uint32_t); j += 4)
                printf("0x%x, 0x%x, 0x%x, 0x%x\n",
                       bigDataLocalPtr[j], bigDataLocalPtr[j + 1], bigDataLocalPtr[j + 2], bigDataLocalPtr[j + 3]);
#endif
            /* Check values to see expected results */
            // for (j = 0; j < shmem->Buf_size; j++)
            // {
            //     // if (shmem->dataBuf[0][j] != (msg->id + 10 + j))
            //     // {
            //     //     errorCount++;
            //     // }
            // }

            // printf("BIGDATA:%f\n", shmem->dataBuf[0][1]);

            /* Free big data buffer */
            // HeapMem_free(sr1Heap, bigDataLocalPtr, bigDataLocalDesc.size);
        }

        printf("App_exec: message received %d\n", msg->id);

        /* free the message */
        MessageQ_free((MessageQ_Msg)msg);

        printf("App_exec: Preparing message %d\n", i);

        /* Receive messages: End =======================================> */

        /* Send messages: Start  <======================================= */

        /* allocate message */
        msg = (App_Msg *)MessageQ_alloc(Module.heapId, Module.msgSize);

        if (msg == NULL)
        {
            status = -1;
            printf("MessageQ_alloc failed\n");
            goto leave;
        }

        /* set the return address in the message header */
        MessageQ_setReplyQueue(Module.hostQue, (MessageQ_Msg)msg);

        /* fill in message payload */
        if (i < 14)
        {

            /* Send Big data messages */

            msg->cmd = App_CMD_BIGDATA;
            msg->id = i;

            /* Allocate buffer from HeapMem */
            // bigDataLocalPtr = (UInt32 *)(HeapMem_alloc(sr1Heap, BIGDATA_BUF_SIZE, BIGDATA_BUF_ALIGN));

            if (!shmem)
            {
                status = -1;
                printf("HeapMem_alloc failed\n");
                goto leave;
            }

            /* Fill Big data buffer */
            // for(j=0; j< BIGDATA_BUF_SIZE/sizeof(uint32_t); j++) {
            //    bigDataLocalPtr[j] = j+i;
            // }

            /* Populate the Local descriptor */
            bigDataLocalDesc.localPtr = (Ptr)shmem;
            bigDataLocalDesc.size = sizeof(Shared_mem);

            retVal = bigDataXlatetoGlobalAndSync(regionId,
                                                 &bigDataLocalDesc, &msg->u.bigDataSharedDesc);
            if (retVal)
            {
                status = -1;
                printf("bigDataXlatetoGlobalAndSync failed\n");
                goto leave;
            }
            msg->regionId = regionId;
        }
        else
        {
            if (i == 16)
            // if (0)
            {
                /* Last message will tell the slave to shutdown */
                msg->cmd = App_CMD_SHUTDOWN;
                msg->id = i;
            }
            else
            {
                /* Send dummy NOP messages before shutdown */
                msg->cmd = App_CMD_NOP;
                msg->id = i;
            }
        }

        printf("App_exec: Sending message %d\n", i);

        /* send message */
        MessageQ_put(Module.slaveQue, (MessageQ_Msg)msg);

        /* Send messages: End  =======================================> */
    }

    /* drain process pipeline */
    for (i = 1; i <= 3; i++)
    {

        /* wait for return message */
        status = MessageQ_get(Module.hostQue, (MessageQ_Msg *)&msg,
                              MessageQ_FOREVER);

        if (status < 0)
        {
            printf("MessageQ_get failed\n");
            goto leave;
        }
        printf("App_exec: message received: %d\n", msg->id);

        /* extract message payload */

        /* free the message */
        MessageQ_free((MessageQ_Msg)msg);
    }

    // cache to local before callback
    retVal = bigDataXlatetoLocalAndSync(msg->regionId,
                                        &msg->u.bigDataSharedDesc, &bigDataLocalDesc);
    if (retVal)
    {
        status = -1;
        return paContinue;
    }
    shmem = (Shared_mem *)bigDataLocalDesc.localPtr;

    // zero shared memory
    for (i = 0; i < FRAMES_PER_BUFFER; i++)
    {
        for (j = 0; j < N_BUFFS; j++)
        {
            // shmem->dataBuf[j][i] = 0;
        }
    }

    if (AudioON)
    {
        startAudio();
    }

    //ui_menu();
    getchar();

    double t = (double)cnt;
    average = average / t;
    printf("ARM->DSP->ARM stats:\nAverage: %f\nMax: %f\n", average, max);

    // drain rest of Messages
    int tmp = 0;

    printf("N unfreed messages %d!\n", MessageQ_count(Module.hostQue));

    // while (tmp != msgIds)
    // {
    //     status = MessageQ_get(Module.hostQue, (MessageQ_Msg *)&msg,
    //                           1000);

    //     if(status >= 0)
    //     {
    //         printf("Freed message id: %d!",msg->id);
    //         tmp = msg->id;
    //         MessageQ_free((MessageQ_Msg)msg);
    //     }
    //     usleep(10);

    // }

leave:
    if (sr1Heap)
    {
        HeapMem_delete(&sr1Heap);
    }
    if (sharedRegionAllocPtr)
    {
        /* free the message */
        CMEM_free(sharedRegionAllocPtr, &cmemAttrs);
    }
    /* Print error count if non-zero */
    if (errorCount)
    {
        printf("App_exec: Error Count %d\n", errorCount);
        status = -1;
    }
    else
        printf("App_exec: Data check clean\n");

    printf("<-- App_exec: %d\n", status);
    return (status);
}
