#include <math.h>
#include <stdlib.h>
#include "reverb.h"
#include "delay.h"

#define SHORT_DL 1024
#define MEDIUM_DL 4096
#define LONG_DL 8192

// reverb variables and structs
static float mono = 0,
             sampleRate = 44100,
             tank1 = 0, tank2 = 0,
             tmp, v = 0, d_v1 = 0, d_v2 = 0;
static double accu;

static varesky_delay in_diff1, in_diff2, in_diff3, in_diff4,
    dec_diff1_1, dec_diff1_2,
    dec_diff2_1, dec_diff2_2;

static varesky_delay predelay,
    tank1_del1, tank2_del1,
    tank1_del2, tank2_del2;

static varesky_lfo s_lfo1, s_lfo2;

static float in_diff_buf1[MEDIUM_DL] = {0},
             in_diff_buf2[MEDIUM_DL] = {0},
             in_diff_buf3[MEDIUM_DL] = {0},
             in_diff_buf4[MEDIUM_DL] = {0},

             dec_diff_buf1_1[LONG_DL] = {0},
             dec_diff_buf1_2[LONG_DL] = {0},
             dec_diff_buf2_1[LONG_DL] = {0},
             dec_diff_buf2_2[LONG_DL] = {0},

             tank1_del1_buf[LONG_DL] = {0},
             tank2_del1_buf[LONG_DL] = {0},
             tank1_del2_buf[LONG_DL] = {0},
             tank2_del2_buf[LONG_DL] = {0},

             predelay_buf[LONG_DL] = {0},

             lfo1, lfo2;

// user parameters/////////////////////////////////////////////////////////////

static float bandwidth = 1, /// 0.89,
             decay = 0.87,
             damping = 0.31,
             predelay_time = 3.11, // predelay time in ms
    mix = 0.5;
///////////////////////////////////////////////////////////////////////////////

float ms2samps(float ms)
{
    return ms * sampleRate * 0.001;
}
void varesky_plate_set_bandwidth(float val)
{
    bandwidth = val;
}
void varesky_plate_set_damping(float val)
{
    damping = val;
}
void varesky_plate_set_decay(float val)
{
    decay = val;
}
void varesky_plate_set_mix(float val)
{
    mix = val;
}
void varesky_plate_set_predelay(float val)
{
    predelay_time = val;
}

void varesky_plate_init(float samplerate)
{
    sampleRate = samplerate;

    varesky_del_setup(&in_diff1, in_diff_buf1, MEDIUM_DL, ms2samps(4.771));
    varesky_del_setup(&in_diff2, in_diff_buf2, MEDIUM_DL, ms2samps(3.595));
    varesky_del_setup(&in_diff3, in_diff_buf3, MEDIUM_DL, ms2samps(12.73));
    varesky_del_setup(&in_diff4, in_diff_buf4, MEDIUM_DL, ms2samps(9.307));

    varesky_del_setup(&dec_diff1_1, dec_diff_buf1_1, LONG_DL, 672); // modulated
    varesky_del_setup(&dec_diff1_2, dec_diff_buf1_2, LONG_DL, ms2samps(89.24));
    varesky_del_setup(&dec_diff2_1, dec_diff_buf2_1, LONG_DL, 1800); // modulated
    varesky_del_setup(&dec_diff2_2, dec_diff_buf2_2, LONG_DL, ms2samps(60.48));

    v_lfo_init(&s_lfo1, samplerate, 1, 0);
    v_lfo_init(&s_lfo2, samplerate, 0.7, 0);

    varesky_del_setup(&predelay, predelay_buf, 4096, 3000);

    varesky_del_setup(&tank1_del1, tank1_del1_buf, LONG_DL, ms2samps(125));
    varesky_del_setup(&tank1_del2, tank1_del2_buf, LONG_DL, ms2samps(141.69));

    varesky_del_setup(&tank2_del1, tank2_del1_buf, LONG_DL, ms2samps(106.28));
    varesky_del_setup(&tank2_del2, tank2_del2_buf, LONG_DL, ms2samps(149.62));
}
void varesky_plate_process_block(const float *in, float *out, int framesPerBuffer)
{
    static unsigned int i = 0;
    for (i = 0; i < framesPerBuffer; i++)
    {

        // sum mono
        mono = 0.5 * (in[2 * i] + in[2 * i + 1]);
        tmp = mono;
        // predelay
        // tmp = varesky_del_process(&predelay, tmp);
        varesky_del_write(&predelay, tmp);
        tmp = varesky_del_read(&predelay, ms2samps(predelay_time));

        // bandwidth
        tmp = bandwidth * tmp + (1 - bandwidth) * v;
        v = tmp;
        // tmp = tmp * bandwidth - v * (bandwidth - 1);
        // v = tmp;

        // input diffusion
        tmp = varesky_ap_process(&in_diff1, tmp, 0.75, ms2samps(4.771));
        tmp = varesky_ap_process(&in_diff2, tmp, 0.75, ms2samps(3.595));
        tmp = varesky_ap_process(&in_diff3, tmp, 0.625, ms2samps(12.73));
        tmp = varesky_ap_process(&in_diff4, tmp, 0.625, ms2samps(9.307));

        tank1 = tmp + decay * varesky_del_read(&tank2_del2, ms2samps(125));    // from tank2
        tank2 = tmp + decay * varesky_del_read(&tank1_del2, ms2samps(106.28)); // from tank1

        // modulated allpases
        lfo1 = v_lfo_process(&s_lfo1);
        lfo2 = v_lfo_process(&s_lfo2);

        tank1 = varesky_ap_process2(&dec_diff1_1, tank1, 0.7, ms2samps(2 * lfo1 + 30.51));
        tank2 = varesky_ap_process2(&dec_diff2_1, tank2, 0.7, ms2samps(2 * lfo2 + 22.58));

        varesky_del_write(&tank1_del1, tank1);
        tank1 = varesky_del_read(&tank1_del1, ms2samps(141.69));

        varesky_del_write(&tank2_del1, tank2);
        tank2 = varesky_del_read(&tank2_del1, ms2samps(149.62));

        // damping
        tank1 = (1 - damping) * tank1 + damping * d_v1;
        d_v1 = tank1;
        tank2 = (1 - damping) * tank2 + damping * d_v2;
        d_v2 = tank2;

        tank1 = varesky_ap_process(&dec_diff1_2, tank1, 0.5, ms2samps(89.24));
        tank2 = varesky_ap_process(&dec_diff2_2, tank2, 0.5, ms2samps(60.48));

        varesky_del_write(&tank1_del2, tank1);
        varesky_del_write(&tank2_del2, tank2);

        accu = varesky_del_read(&tank2_del1, ms2samps(8.9));
        accu -= varesky_del_read(&tank2_del1, ms2samps(99.8));
        accu += varesky_del_read(&dec_diff2_2, ms2samps(64.2));
        accu -= varesky_del_read(&tank2_del2, ms2samps(67));
        accu += varesky_del_read(&tank1_del1, ms2samps(66.8));
        accu -= varesky_del_read(&dec_diff1_2, ms2samps(6.3));
        accu -= varesky_del_read(&tank1_del2, ms2samps(35.8));
        accu *= 0.2;
        out[2 * i] = (1 - mix) * mono + mix * accu;

        accu = varesky_del_read(&tank1_del1, ms2samps(11.8));
        accu -= varesky_del_read(&tank1_del1, ms2samps(121.7));
        accu += varesky_del_read(&dec_diff1_2, ms2samps(41.2));
        accu -= varesky_del_read(&tank1_del2, ms2samps(89.7));
        accu += varesky_del_read(&tank2_del1, ms2samps(70.8));
        accu -= varesky_del_read(&dec_diff2_2, ms2samps(11.2));
        accu -= varesky_del_read(&tank2_del2, ms2samps(4.1));
        accu *= 0.2;
        out[2 * i + 1] = (1 - mix) * mono + mix * accu;
    }
}

void v_lfo_init(varesky_lfo *lfo, float samplerate, float rate, float startPhase)
{
    lfo->rate = rate;
    lfo->state = startPhase;
    lfo->samplerate = samplerate;
    v_lfo_set_rate(lfo,rate);
}
void v_lfo_set_rate(varesky_lfo *lfo, float rate)
{
    //lfo->rate = rate;
    lfo->inc = 2 * M_PI * rate / lfo->samplerate;
}
float v_lfo_process2(varesky_lfo *lfo)
{
    lfo->state += lfo->rate;
    if (lfo->state > 2)
        lfo->state -= 4;
    return lfo->state * (2 - abs(lfo->state));
}
float v_lfo_process(varesky_lfo *lfo)
{

    static float tmp;
    tmp = sin(lfo->state);

    lfo->state += lfo->inc;
    if (lfo->state > 2 * M_PI)
        lfo->state -= 2 * M_PI;

    return tmp;
}
